package eu.jonahbauer.wizard.client.libgdx.actors.game;

import com.badlogic.gdx.scenes.scene2d.*;
import eu.jonahbauer.wizard.client.libgdx.WizardGame;
import eu.jonahbauer.wizard.client.libgdx.screens.GameScreen;
import lombok.Data;

import java.util.*;

import static com.badlogic.gdx.scenes.scene2d.actions.Actions.*;

public class CardStack extends Group {
    private final Random random = new Random();
    private final List<Entry> cards = new ArrayList<>();

    public CardStack() {
        this.addListener(new InputListener() {
            @Override
            public void enter(InputEvent event, float x, float y, int pointer, Actor fromActor) {
                if (fromActor == null || !isAscendantOf(fromActor)) {
                    cards.forEach(Entry::expand);
                }
            }

            @Override
            public void exit(InputEvent event, float x, float y, int pointer, Actor toActor) {
                if (toActor == null || !isAscendantOf(toActor)) {
                    cards.forEach(Entry::collapse);
                }
            }
        });
    }

    public void add(GameScreen.Seat seat, CardActor card) {
        var entry = new Entry(
                card,
                seat,
                (float) random.nextGaussian((WizardGame.WIDTH - card.getWidth()) / 2, 15),
                (float) random.nextGaussian((WizardGame.HEIGHT - card.getHeight()) / 2, 15),
                (float) random.nextGaussian(0, 60),
                (float) random.nextGaussian(0, 20)
        );
        addActor(card);
        entry.collapse();
        cards.add(entry);
    }

    @Override
    public void clearChildren(boolean unfocus) {
        super.clearChildren(unfocus);
        cards.clear();
    }

    @Data
    private static class Entry {
        private final CardActor actor;
        private final GameScreen.Seat seat;
        private final float x;
        private final float y;
        private final float rotation;
        private final float rotation2;
        private Action action;

        public void expand() {
            if (action != null) {
                actor.removeAction(action);
            }

            action = parallel(
                    moveTo(seat.getCardX() - actor.getWidth() / 2, seat.getCardY() - actor.getHeight() / 2, 0.25f),
                    rotateTo(rotation2, 0.25f)
            );

            actor.addAction(action);
        }

        public void collapse() {
            if (action != null) {
                actor.removeAction(action);
            }

            action = parallel(
                    moveTo(x, y, 0.25f),
                    rotateTo(rotation, 0.25f)
            );

            actor.addAction(action);
        }
    }
}
